﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Media;
using Buddy.Coroutines;
using ff14bot;
using ff14bot.AClasses;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Objects;
using ff14bot.RemoteWindows;
using Newtonsoft.Json;
using TreeSharp;
using System.ComponentModel;

namespace TargetWeakestEnemyPlugin
{
    // ReSharper disable once UnusedMember.Global
    public class TargetWeakestEnemyPlugin : BotPlugin
    {
        #region Properties

        private static BattleCharacter weakestEnemy;
        private static Stopwatch timer = new Stopwatch();

        private static bool CheckWeakest
        {
            get
            {
                try
                {
                    if (Core.Me.CurrentTarget == null || Core.Me.CurrentTarget.Type != GameObjectType.BattleNpc ||
                        Poi.Current.Type != PoiType.Kill ||
                        !GameObjectManager.Attackers.Contains((BattleCharacter)Core.Me.CurrentTarget))
                        return false;
                    else weakestEnemy = (BattleCharacter)Core.Me.CurrentTarget;

                    foreach (var enemy in GameObjectManager.Attackers)
                    {
                        if (enemy.IsAlive && enemy.CanAttack && enemy.IsTargetable)
                        {
                            if (Settings.Instance.CheckSpecificTargets)
                            {
                                if (enemy != null && enemy.CurrentTargetId == Core.Player.ObjectId)
                                {
                                    if (weakestEnemy != null && enemy != null && enemy.CurrentHealth < weakestEnemy.CurrentHealth) weakestEnemy = enemy;
                                }
                            }
                            else
                            {
                                if (weakestEnemy != null && enemy != null && enemy.CurrentHealth < weakestEnemy.CurrentHealth) weakestEnemy = enemy;
                            }
                        }
                    }

                    if (weakestEnemy != null && Core.Me.CurrentTarget != weakestEnemy && (!timer.IsRunning || timer.ElapsedMilliseconds > (Settings.Instance.SecondsBetweenTargetChanges * 1000)))
                    {
                        timer.Restart();
                        return true;
                    }
                    return false;
                }
                catch (Exception e)
                {
                    Logging.Write(Colors.Aqua, "[TargetWeakestEnemy] CheckWeakest exception encountered." + Environment.NewLine + e.ToString());
                    return false;
                }
            }
        }

        #endregion

        #region Target Enemy Logic

        private static async Task<bool> TargetWeakest()
        {
            try
            {
                if (FishingManager.State != FishingState.None)
                {
                    Logging.Write(Colors.Aqua, "[TargetWeakestEnemy] Currently Fishing, waiting until we are done.");
                    return false;
                }

                if (GatheringManager.WindowOpen)
                {
                    Logging.Write(Colors.Aqua, "[TargetWeakestEnemy] Currently Gathering Items, waiting until we are done.");
                    return false;
                }

                if (CraftingManager.IsCrafting)
                {
                    Logging.Write(Colors.Pink, "[TargetWeakestEnemy] Currently Crafting Items, waiting until we are done.");
                    return false;
                }

                if (CraftingLog.IsOpen)
                {
                    Logging.Write(Colors.Pink, "[TargetWeakestEnemy] Currently Crafting Items, waiting until we are done.");
                    return false;
                }

                if (Core.Me.IsMounted)
                {
                    Logging.Write(Colors.Aqua, "[TargetWeakestEnemy] Currently Mounted, waiting until we are dismounted.");
                    return false;
                }

                await TargetWeakestNow();

                return false;
            }
            catch (Exception e)
            {
                Logging.Write(Colors.Aqua, "[TargetWeakestEnemy] TargetWeakest exception encountered." + Environment.NewLine + e.ToString());
                return false;
            }
        }

        private static async Task<bool> TargetWeakestNow()
        {
            if (weakestEnemy != null && Core.Me.CurrentTarget != null && Core.Me.CurrentTarget != weakestEnemy && Poi.Current.Type == PoiType.Kill)
            {
                if (Debugger.IsAttached) Debugger.Break();
                Logging.Write(Colors.Aqua, "[TargetWeakestEnemy] Now targeting " + weakestEnemy.EnglishName);
                //Poi.Clear("Clearing Poi to retarget weakest");
                //Core.Me.ClearTarget();
                if (weakestEnemy != null) Poi.Current = new Poi(weakestEnemy, PoiType.Kill);
                if (weakestEnemy != null) weakestEnemy.Target();
            }
            return false;
        }

        #endregion

            #region Fields

        private Composite _coroutine;
        private Gui _settingsForm;

        #endregion

        #region Required Overrides

        public override string Author
        {
            get { return "Cloud30000"; }
        }

        public override string Name
        {
            get { return "TargetWeakestEnemy"; }
        }

        public override Version Version
        {
            get { return new Version(1, 0, 0, 0); }
        }

        #endregion

        #region Optional Overrides

        public override void OnInitialize()
        {
            _coroutine = new Decorator(c => Core.Me.IsAlive && Core.Me.InCombat && !Core.Me.IsMounted && CheckWeakest, new ActionRunCoroutine(r => TargetWeakest()));
        }

        public override void OnEnabled()
        {
            TreeRoot.OnStart += OnBotStart;
            TreeRoot.OnStop += OnBotStop;
            TreeHooks.Instance.OnHooksCleared += OnOnHooksCleared;

            if (TreeRoot.IsRunning)
            {
                AddHooks();
            }
        }

        public override void OnDisabled()
        {
            TreeRoot.OnStart -= OnBotStart;
            TreeRoot.OnStop -= OnBotStop;
            TreeHooks.Instance.OnHooksCleared -= OnOnHooksCleared;
            RemoveHooks();
        }

        public override void OnShutdown()
        {
            TreeRoot.OnStart -= OnBotStart;
            TreeRoot.OnStop -= OnBotStop;
            TreeHooks.Instance.OnHooksCleared -= OnOnHooksCleared;
            RemoveHooks();
        }

        public override bool WantButton
        {
            get { return true; }
        }

        public override void OnButtonPress()
        {
            if (_settingsForm == null || _settingsForm.IsDisposed || _settingsForm.Disposing)
                _settingsForm = new Gui();

            _settingsForm.ShowDialog();
        }

        #endregion

        #region Hook Logic

        private void AddHooks()
        {
            Logging.Write(Colors.Aqua, "Adding TargetWeakestEnemy Hook");
            TreeHooks.Instance.InsertHook("TreeStart", 0, _coroutine);
        }

        private void RemoveHooks()
        {
            Logging.Write(Colors.Aqua, "Removing TargetWeakestEnemy Hook");
            TreeHooks.Instance.RemoveHook("TreeStart", _coroutine);
        }

        #endregion

        #region Event Handlers

        private void OnBotStop(BotBase bot)
        {
            RemoveHooks();
        }

        private void OnBotStart(BotBase bot)
        {
            AddHooks();
        }

        private void OnOnHooksCleared(object sender, EventArgs e)
        {
            AddHooks();
        }

        #endregion
    }

    #region Extensions Class

    public static class Extensions
    {
        public static IEnumerable<BagSlot> MedicineItems(this IEnumerable<BagSlot> inventory)
        {
            return inventory.Where(x => x.MedicineItem());
        }

        public static IEnumerable<BagSlot> InventoryItems(this IEnumerable<BagSlot> inventory)
        {
            return
                inventory.Where(
                    x =>
                        x.BagId == InventoryBagId.Bag1 || x.BagId == InventoryBagId.Bag2 ||
                        x.BagId == InventoryBagId.Bag3 || x.BagId == InventoryBagId.Bag4);
        }

        private static bool MedicineItem(this BagSlot slot)
        {
            return (slot.Item.EquipmentCatagory == ItemUiCategory.Medicine) && (int)slot.Item.ItemRole == 6;
        }
    }

    #endregion

    #region Settings Class

    public class Settings : JsonSettings
    {
        [JsonIgnore]
        private static Settings _instance;
        // ReSharper disable once UnusedParameter.Local
        // ReSharper disable once MemberCanBePrivate.Global
        public Settings(string filename)
            : base(Path.Combine(CharacterSettingsDirectory, "TargetWeakestEnemy.json"))
        {
        }

        public static Settings Instance
        {
            get { return _instance ?? (_instance = new Settings("Settings")); }
        }

        [Setting]
        [DefaultValue(true)]
        public bool CheckSpecificTargets { get; set; }

        [Setting]
        [DefaultValue(6)]
        public int SecondsBetweenTargetChanges { get; set; }
    }

    #endregion
}